@@ -1,5 +1,36 @@
 import pandas as pd
 from collections.abc import Iterable
+from pathlib import Path
+from fnmatch import fnmatch
+import os
+
+def read_directory(dirname, read=pd.read_csv,
+                   # fnmatch filters for the filename
+                   fn_match=None, fn_not_match=None,
+                   # Take parameters from the filename
+                   fn_cols=(), fn_prefix='file_',
+                   # Arguments for the read function
+                   **kwargs):
+    dfs = []
+    for fn in Path(os.path.expanduser(dirname)).iterdir():
+        if fn_match is not None and not fnmatch.fnmatch(fn, fn_match):
+            continue
+        if fn_not_match is not None and fnmatch.fnmatch(fn, fn_not_match):
+            continue
+
+        # Read the Data
+        df =  read(fn, **kwargs)
+        # Remove all spaces in column headers, nobody needs spaces
+        df.columns = df.columns.str.strip()
+
+        # Import Columns from Path object
+        for col in fn_cols:
+            df[fn_prefix + col] = getattr(fn, col)
+
+        dfs.append(df)
+
+    return pd.concat(dfs)
+
 
 def missing_rows(data, collapse=True, known_missing=None):
     """Find missing measurements in large datasets.
@@ -123,10 +154,10 @@ def select_quantiles(df, q=[0.0, 0.5, 1.0], columns=None, compress=True,
         value_cols = False
 
     # Get those rows from df that are at the quantile q in the column col.
-    def get_rows(col, q):
-        idx = int(q * (len(df)-1))
-        val = df.iloc[idx][col]
-        rows = df[df[col] == val].copy()
+    def get_rows(df_sort, col, q):
+        idx = int(q * (len(df_sort)-1))
+        val = df_sort.iloc[idx][col]
+        rows = df_sort[df_sort[col] == val].copy()
         if compress:
             cardinality = len(rows)
             rows = rows.iloc[:1]
@@ -153,9 +184,9 @@ def select_quantiles(df, q=[0.0, 0.5, 1.0], columns=None, compress=True,
     ret = []
     columns = columns or df.columns
     for col in columns:
-        df = df.sort_values(col)
+        df_sort = df[df[col].notna()].sort_values(col)
         for _q in q:
-            ret += [get_rows(col, _q)]
+            ret += [get_rows(df_sort, col, _q)]
 
     # Concatenate the individual rows
     ret = pd.concat(ret).set_index([column_col, q_col])
@@ -166,7 +197,8 @@ def select_quantiles(df, q=[0.0, 0.5, 1.0], columns=None, compress=True,
         if  isinstance(index_cols, Iterable):
             ordered += list(index_cols)
         else:
-            ordered += list(df.index.names)
+            ordered += list([n for n in df.index.names if n])
+
     if value_cols:
         if  isinstance(value_cols, Iterable):
             ordered += list(value_cols)
@@ -177,3 +209,45 @@ def select_quantiles(df, q=[0.0, 0.5, 1.0], columns=None, compress=True,
 
     return ret[ordered]
 
+
+def reorder_by(column, categories, dropna=True):
+    """Returns a function that reorders an column as an ordered category:
+
+       Can be used with plydata:
+
+       df >> do(reorder_by('foo', ['a, 'c', 'b']))"""
+    def helper(df):
+        df = df.copy()
+        df[column] = df[column].astype('category').cat.set_categories(categories, ordered=True)
+        if dropna:
+            df = df[df[column].notna()]
+        return df
+    return helper
+
+
+def mapvalues(column, keys=None, values=None, na_action=None, keep_order=True, **kwargs):
+    """Returns a function that:
+       1. Takes an DataFrame
+       2. Selects the given column
+       3. Maps the column-contents according to key->value mapping (or kwargs dict)
+       4. Returns a pandas.Series
+
+    Can be used with plydata:
+
+    >>> df >> define(xyz_label=mapvalues('xyz', x='System 1', y='System 2'))
+    """
+    translate_dict = None
+    if keys is not None:
+        assert values is not None and len(keys) == len(values)
+        translate_dict = dict(zip(keys, values))
+    else:
+        assert kwargs is not None
+        translate_dict = kwargs
+
+    def mapper(df):
+        series = df[column].map(translate_dict, na_action=na_action)
+        if values is not None and keep_order:
+            series = series.astype('category').cat.set_categories(values, ordered=True)
+        return series
+    return mapper
+
